declare abstract class ToolboxPanel {
    title: string;
    icon: string;
    constructor(title: string, icon?: string);
    abstract getUi(): HTMLDivElement;
}

interface MarkerBaseState {
    typeName: string;
    state: MarkerState;
}

declare type ColorSet = string[];
declare type DisplayMode = 'inline' | 'popup';
declare class Settings {
    defaultColorSet: ColorSet;
    defaultColor: string;
    defaultFillColor: string;
    defaultStrokeColor: string;
    defaultHighlightColor: string;
    defaultStrokeWidth: number;
    defaultStrokeDasharray: string;
    defaultHighlightOpacity: number;
    defaultFontFamily: string;
    defaultStrokeWidths: number[];
    defaultStrokeDasharrays: string[];
    defaultOpacitySteps: number[];
    displayMode: DisplayMode;
    defaultFontFamilies: string[];
}

declare type MarkerState = 'new' | 'creating' | 'select' | 'move' | 'resize' | 'rotate';
declare class MarkerBase {
    static typeName: string;
    protected _container: SVGGElement;
    get container(): SVGGElement;
    protected _overlayContainer: HTMLDivElement;
    get overlayContainer(): HTMLDivElement;
    protected _state: MarkerState;
    get state(): MarkerState;
    protected globalSettings: Settings;
    get toolboxPanels(): ToolboxPanel[];
    static title: string;
    /**
     * SVG icon markup
     */
    static icon: string;
    onMarkerCreated: (marker: MarkerBase) => void;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    select(): void;
    deselect(): void;
    pointerDown(point: IPoint, target?: EventTarget): void;
    dblClick(point: IPoint, target?: EventTarget): void;
    manipulate(point: IPoint): void;
    pointerUp(point: IPoint): void;
    dispose(): void;
    protected addMarkerVisualToContainer(element: SVGElement): void;
    getState(): MarkerBaseState;
    restoreState(state: MarkerBaseState): void;
}

interface MarkerAreaState {
    markers: MarkerBaseState[];
}

interface IStyleSettings {
    toolbarBackgroundColor?: string;
    toolbarBackgroundHoverColor?: string;
    toolbarColor?: string;
    toolbarHeight?: number;
    toolboxBackgroundColor?: string;
    toolboxColor?: string;
    toolboxAccentColor?: string;
    toolbarStyleColorsClassName?: string;
    toolbarOverflowBlockStyleColorsClassName?: string;
    toolbarButtonStyleColorsClassName?: string;
    toolbarActiveButtonStyleColorsClassName?: string;
    toolboxStyleColorsClassName?: string;
    toolboxButtonStyleColorsClassName?: string;
    toolboxActiveButtonStyleColorsClassName?: string;
    toolboxButtonRowStyleColorsClassName?: string;
    toolboxPanelRowStyleColorsClassName?: string;
}

declare type MarkerTypeIdentifier = string | typeof MarkerBase;
interface IPoint {
    x: number;
    y: number;
}
declare type RenderEventHandler = (dataURL: string, state?: MarkerAreaState) => void;
declare type CloseEventHandler = () => void;
declare class MarkerArea {
    private target;
    private targetRoot;
    private width;
    private height;
    private left;
    private top;
    private markerImage;
    private markerImageHolder;
    private defs;
    private coverDiv;
    private uiDiv;
    private contentDiv;
    private editorCanvas;
    private editingTarget;
    private overlayContainer;
    private logoUI;
    get ALL_MARKER_TYPES(): typeof MarkerBase[];
    get DEFAULT_MARKER_TYPES(): typeof MarkerBase[];
    get BASIC_MARKER_TYPES(): typeof MarkerBase[];
    private _availableMarkerTypes;
    get availableMarkerTypes(): MarkerTypeIdentifier[];
    set availableMarkerTypes(value: MarkerTypeIdentifier[]);
    private toolbar;
    private toolbox;
    private mode;
    private currentMarker?;
    private markers;
    private isDragging;
    private bodyOverflowState;
    private scrollYState;
    private scrollXState;
    private renderEventListeners;
    private closeEventListeners;
    settings: Settings;
    uiStyleSettings: IStyleSettings;
    constructor(target: HTMLImageElement);
    private open;
    show(): void;
    render(): Promise<string>;
    close(): void;
    addMarkersToToolbar(...markers: typeof MarkerBase[]): void;
    addRenderEventListener(listener: RenderEventHandler): void;
    removeRenderEventListener(listener: RenderEventHandler): void;
    addCloseEventListener(listener: CloseEventHandler): void;
    removeCloseEventListener(listener: CloseEventHandler): void;
    private setEditingTarget;
    private setTopLeft;
    private initMarkerCanvas;
    private initOverlay;
    private positionMarkerImage;
    private attachEvents;
    /**
     * NOTE:
     *
     * before removing or modifying this method please consider supporting marker.js
     * by visiting https://markerjs.com/#price for details
     *
     * thank you!
     */
    private addLogo;
    private positionLogo;
    private overrideOverflow;
    private restoreOverflow;
    private showUI;
    private closeUI;
    private toolbarButtonClicked;
    private renderClicked;
    getState(): MarkerAreaState;
    restoreState(state: MarkerAreaState): void;
    private addNewMarker;
    private createNewMarker;
    private markerCreated;
    private setCurrentMarker;
    private onPointerDown;
    private onDblClick;
    private onPointerMove;
    private onPointerUp;
    private clientToLocalCoordinates;
    private onWindowResize;
    private positionUI;
}

declare class Activator {
    private static key;
    static addKey(key: string): void;
    static get isLicensed(): boolean;
}

declare class SvgHelper {
    static createDefs(): SVGDefsElement;
    static setAttributes(el: SVGElement, attributes: Array<[string, string]>): void;
    static createRect(width: number | string, height: number | string, attributes?: Array<[string, string]>): SVGRectElement;
    static createLine(x1: number | string, y1: number | string, x2: number | string, y2: number | string, attributes?: Array<[string, string]>): SVGLineElement;
    static createPolygon(points: string, attributes?: Array<[string, string]>): SVGPolygonElement;
    static createCircle(radius: number, attributes?: Array<[string, string]>): SVGCircleElement;
    static createEllipse(rx: number, ry: number, attributes?: Array<[string, string]>): SVGEllipseElement;
    static createGroup(attributes?: Array<[string, string]>): SVGGElement;
    static createTransform(): SVGTransform;
    static createMarker(id: string, orient: string, markerWidth: number | string, markerHeight: number | string, refX: number | string, refY: number | string, markerElement: SVGGraphicsElement): SVGMarkerElement;
    static createText(attributes?: Array<[string, string]>): SVGTextElement;
    static createTSpan(text: string, attributes?: Array<[string, string]>): SVGTSpanElement;
    static createImage(attributes?: Array<[string, string]>): SVGImageElement;
    static createPoint(x: number, y: number): SVGPoint;
}

declare class Style {
    static CLASS_PREFIX: string;
    private static classes;
    private static rules;
    private static styleSheet?;
    static settings: IStyleSettings;
    static get fadeInAnimationClassName(): string;
    static get fadeOutAnimationClassName(): string;
    static addClass(styleClass: StyleClass): StyleClass;
    static addRule(styleRule: StyleRule): void;
    private static addStyleSheet;
}
declare class StyleRule {
    selector: string;
    style: string;
    constructor(selector: string, style: string);
}
declare class StyleClass {
    style: string;
    private _localName;
    get name(): string;
    constructor(name: string, style: string);
}

interface ITransformMatrix {
    a: number;
    b: number;
    c: number;
    d: number;
    e: number;
    f: number;
}
declare class TransformMatrix {
    static toITransformMatrix(matrix: SVGMatrix): ITransformMatrix;
    static toSVGMatrix(currentMatrix: SVGMatrix, newMatrix: ITransformMatrix): SVGMatrix;
}

declare type ArrowType = 'both' | 'start' | 'end' | 'none';
declare type ArrowTypeChangeHandler = (newType: ArrowType) => void;
declare class ArrowTypePanel extends ToolboxPanel {
    private currentType?;
    private typeBoxes;
    onArrowTypeChanged?: ArrowTypeChangeHandler;
    constructor(title: string, currentType?: ArrowType, icon?: string);
    getUi(): HTMLDivElement;
    private setCurrentType;
}

declare type ColorChangeHandler = (newColor: string) => void;
declare class ColorPickerPanel extends ToolboxPanel {
    private colors;
    private currentColor?;
    private colorBoxes;
    onColorChanged?: ColorChangeHandler;
    constructor(title: string, colors: string[], currentColor?: string, icon?: string);
    getUi(): HTMLDivElement;
    private setCurrentColor;
}

declare type FontChangeHandler = (newFont: string) => void;
declare class FontFamilyPanel extends ToolboxPanel {
    private fonts;
    private currentFont?;
    private fontBoxes;
    onFontChanged?: FontChangeHandler;
    constructor(title: string, fonts: string[], currentFont?: string, icon?: string);
    getUi(): HTMLDivElement;
    private setCurrentFont;
}

declare type StyleChangeHandler = (newStyle: string) => void;
declare class LineStylePanel extends ToolboxPanel {
    private styles;
    private currentStyle?;
    private styleBoxes;
    onStyleChanged?: StyleChangeHandler;
    constructor(title: string, styles: string[], currentStyle?: string, icon?: string);
    getUi(): HTMLDivElement;
    private setCurrentStyle;
}

declare type WidthChangeHandler = (newWidth: number) => void;
declare class LineWidthPanel extends ToolboxPanel {
    private widths;
    private currentWidth?;
    private widthBoxes;
    onWidthChanged?: WidthChangeHandler;
    constructor(title: string, widths: number[], currentWidth?: number, icon?: string);
    getUi(): HTMLDivElement;
    private setCurrentWidth;
}

declare type OpacityChangeHandler = (newOpacity: number) => void;
declare class OpacityPanel extends ToolboxPanel {
    private opacities;
    private currentOpacity?;
    private opacityBoxes;
    onOpacityChanged?: OpacityChangeHandler;
    constructor(title: string, opacities: number[], currentOpacity?: number, icon?: string);
    getUi(): HTMLDivElement;
    private setCurrentOpacity;
}

interface LinearMarkerBaseState extends MarkerBaseState {
    x1: number;
    y1: number;
    x2: number;
    y2: number;
}

declare class LinearMarkerBase extends MarkerBase {
    protected x1: number;
    protected y1: number;
    protected x2: number;
    protected y2: number;
    protected defaultLength: number;
    protected manipulationStartX: number;
    protected manipulationStartY: number;
    protected manipulationStartX1: number;
    protected manipulationStartY1: number;
    protected manipulationStartX2: number;
    protected manipulationStartY2: number;
    protected visual: SVGGraphicsElement;
    private controlBox;
    private grip1;
    private grip2;
    private activeGrip;
    protected markerElement: SVGGElement;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    pointerDown(point: IPoint, target?: EventTarget): void;
    pointerUp(point: IPoint): void;
    protected adjustVisual(): void;
    manipulate(point: IPoint): void;
    protected resize(point: IPoint): void;
    select(): void;
    deselect(): void;
    private setupControlBox;
    private adjustControlBox;
    private addControlGrips;
    private createGrip;
    private positionGrips;
    private positionGrip;
    getState(): LinearMarkerBaseState;
    restoreState(state: MarkerBaseState): void;
}

interface RectangularBoxMarkerBaseState extends MarkerBaseState {
    left: number;
    top: number;
    width: number;
    height: number;
    rotationAngle: number;
    visualTransformMatrix: ITransformMatrix;
    containerTransformMatrix: ITransformMatrix;
}

declare class RectangularBoxMarkerBase extends MarkerBase {
    protected left: number;
    protected top: number;
    protected width: number;
    protected height: number;
    protected defaultSize: IPoint;
    protected manipulationStartLeft: number;
    protected manipulationStartTop: number;
    protected manipulationStartWidth: number;
    protected manipulationStartHeight: number;
    protected manipulationStartX: number;
    protected manipulationStartY: number;
    protected offsetX: number;
    protected offsetY: number;
    protected rotationAngle: number;
    protected get centerX(): number;
    protected get centerY(): number;
    protected visual: SVGGraphicsElement;
    protected controlBox: SVGGElement;
    private readonly CB_DISTANCE;
    private controlRect;
    private rotatorGripLine;
    private controlGrips;
    private rotatorGrip;
    private activeGrip;
    protected markerElement: SVGGElement;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    pointerDown(point: IPoint, target?: EventTarget): void;
    pointerUp(point: IPoint): void;
    protected moveVisual(point: IPoint): void;
    manipulate(point: IPoint): void;
    protected resize(point: IPoint): void;
    protected setSize(): void;
    private rotate;
    private applyRotation;
    protected rotatePoint(point: IPoint): IPoint;
    protected unrotatePoint(point: IPoint): IPoint;
    select(): void;
    deselect(): void;
    private setupControlBox;
    private adjustControlBox;
    private addControlGrips;
    private createGrip;
    private positionGrips;
    private positionGrip;
    protected hideControlBox(): void;
    protected showControlBox(): void;
    getState(): RectangularBoxMarkerBaseState;
    restoreState(state: MarkerBaseState): void;
}

interface RectangleMarkerState extends RectangularBoxMarkerBaseState {
    fillColor: string;
    strokeColor: string;
    strokeWidth: number;
    strokeDasharray: string;
    opacity: number;
}

declare abstract class RectangleMarker extends RectangularBoxMarkerBase {
    static title: string;
    protected fillColor: string;
    protected strokeColor: string;
    protected strokeWidth: number;
    protected strokeDasharray: string;
    protected opacity: number;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    protected createVisual(): void;
    pointerDown(point: IPoint, target?: EventTarget): void;
    manipulate(point: IPoint): void;
    protected resize(point: IPoint): void;
    protected setSize(): void;
    pointerUp(point: IPoint): void;
    protected setStrokeColor(color: string): void;
    protected setFillColor(color: string): void;
    protected setStrokeWidth(width: number): void;
    protected setStrokeDasharray(dashes: string): void;
    getState(): RectangleMarkerState;
    restoreState(state: MarkerBaseState): void;
}

declare class ResizeGrip {
    visual: SVGGraphicsElement;
    readonly GRIP_SIZE = 10;
    constructor();
    ownsTarget(el: EventTarget): boolean;
}

declare class RectangularBoxMarkerGrips {
    topLeft: ResizeGrip;
    topCenter: ResizeGrip;
    topRight: ResizeGrip;
    centerLeft: ResizeGrip;
    centerRight: ResizeGrip;
    bottomLeft: ResizeGrip;
    bottomCenter: ResizeGrip;
    bottomRight: ResizeGrip;
    constructor();
    findGripByVisual(gripVisual: EventTarget): ResizeGrip | undefined;
}

interface LineMarkerState extends LinearMarkerBaseState {
    strokeColor: string;
    strokeWidth: number;
    strokeDasharray: string;
}

declare class LineMarker extends LinearMarkerBase {
    static typeName: string;
    static title: string;
    static icon: string;
    protected selectorLine: SVGLineElement;
    protected visibleLine: SVGLineElement;
    protected strokeColor: string;
    protected strokeWidth: number;
    protected strokeDasharray: string;
    protected strokePanel: ColorPickerPanel;
    protected strokeWidthPanel: LineWidthPanel;
    protected strokeStylePanel: LineStylePanel;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    private createVisual;
    pointerDown(point: IPoint, target?: EventTarget): void;
    protected adjustVisual(): void;
    protected setStrokeColor(color: string): void;
    protected setStrokeWidth(width: number): void;
    protected setStrokeDasharray(dashes: string): void;
    get toolboxPanels(): ToolboxPanel[];
    getState(): LineMarkerState;
    restoreState(state: MarkerBaseState): void;
}

interface ArrowMarkerState extends LineMarkerState {
    arrowType: ArrowType;
}

declare class ArrowMarker extends LineMarker {
    static typeName: string;
    static title: string;
    static icon: string;
    private arrow1;
    private arrow2;
    private arrowType;
    private arrowBaseHeight;
    private arrowBaseWidth;
    protected arrowTypePanel: ArrowTypePanel;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    private getArrowPoints;
    private createTips;
    pointerDown(point: IPoint, target?: EventTarget): void;
    protected adjustVisual(): void;
    private setArrowType;
    get toolboxPanels(): ToolboxPanel[];
    getState(): ArrowMarkerState;
    restoreState(state: MarkerBaseState): void;
}

interface TextMarkerState extends RectangularBoxMarkerBaseState {
    color: string;
    fontFamily: string;
    padding: number;
    text: string;
}

declare class TextMarker extends RectangularBoxMarkerBase {
    static typeName: string;
    static title: string;
    static icon: string;
    protected color: string;
    protected fontFamily: string;
    protected padding: number;
    protected colorPanel: ColorPickerPanel;
    protected fontFamilyPanel: FontFamilyPanel;
    private readonly DEFAULT_TEXT;
    private text;
    protected textElement: SVGTextElement;
    protected bgRectangle: SVGRectElement;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    protected createVisual(): void;
    pointerDown(point: IPoint, target?: EventTarget): void;
    private renderText;
    private getTextScale;
    private getTextPosition;
    private sizeText;
    manipulate(point: IPoint): void;
    protected resize(point: IPoint): void;
    protected setSize(): void;
    pointerUp(point: IPoint): void;
    private showTextEditor;
    private textEditDivClicked;
    dblClick(point: IPoint, target?: EventTarget): void;
    protected setColor(color: string): void;
    protected setFont(font: string): void;
    protected hideVisual(): void;
    protected showVisual(): void;
    get toolboxPanels(): ToolboxPanel[];
    getState(): TextMarkerState;
    restoreState(state: MarkerBaseState): void;
}

interface CalloutMarkerState extends TextMarkerState {
    bgColor: string;
    tipPosition: IPoint;
}

declare class CalloutMarker extends TextMarker {
    static typeName: string;
    static title: string;
    static icon: string;
    private bgColor;
    protected bgColorPanel: ColorPickerPanel;
    private tipPosition;
    private tipBase1Position;
    private tipBase2Position;
    private tip;
    private tipGrip;
    private tipMoving;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    private createTip;
    pointerDown(point: IPoint, target?: EventTarget): void;
    pointerUp(point: IPoint): void;
    manipulate(point: IPoint): void;
    protected setBgColor(color: string): void;
    private getTipPoints;
    private setTipPoints;
    protected resize(point: IPoint): void;
    private positionTip;
    get toolboxPanels(): ToolboxPanel[];
    select(): void;
    getState(): CalloutMarkerState;
    restoreState(state: MarkerBaseState): void;
}

declare class CoverMarker extends RectangleMarker {
    static title: string;
    static icon: string;
    protected fillPanel: ColorPickerPanel;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    get toolboxPanels(): ToolboxPanel[];
}

declare class EllipseMarker extends RectangularBoxMarkerBase {
    static typeName: string;
    static title: string;
    static icon: string;
    protected fillColor: string;
    protected strokeColor: string;
    protected strokeWidth: number;
    protected strokeDasharray: string;
    protected opacity: number;
    private strokePanel;
    private fillPanel;
    private strokeWidthPanel;
    private strokeStylePanel;
    protected opacityPanel: OpacityPanel;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    protected createVisual(): void;
    pointerDown(point: IPoint, target?: EventTarget): void;
    manipulate(point: IPoint): void;
    protected resize(point: IPoint): void;
    protected setSize(): void;
    pointerUp(point: IPoint): void;
    protected setStrokeColor(color: string): void;
    protected setFillColor(color: string): void;
    protected setStrokeWidth(width: number): void;
    protected setStrokeDasharray(dashes: string): void;
    protected setOpacity(opacity: number): void;
    get toolboxPanels(): ToolboxPanel[];
    getState(): RectangleMarkerState;
    restoreState(state: MarkerBaseState): void;
}

declare class FrameMarker extends RectangleMarker {
    static typeName: string;
    static title: string;
    static icon: string;
    private strokePanel;
    private strokeWidthPanel;
    private strokeStylePanel;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    get toolboxPanels(): ToolboxPanel[];
    getState(): RectangleMarkerState;
}

interface FreehandMarkerState extends RectangularBoxMarkerBaseState {
    drawingImgUrl: string;
}

declare class FreehandMarker extends RectangularBoxMarkerBase {
    static typeName: string;
    static title: string;
    static icon: string;
    protected color: string;
    protected lineWidth: number;
    private colorPanel;
    private canvasElement;
    private canvasContext;
    private drawingImage;
    private drawingImgUrl;
    private drawing;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    private createVisual;
    pointerDown(point: IPoint, target?: EventTarget): void;
    manipulate(point: IPoint): void;
    protected resize(point: IPoint): void;
    pointerUp(point: IPoint): void;
    private addCanvas;
    select(): void;
    deselect(): void;
    private finishCreation;
    private setDrawingImage;
    protected setColor(color: string): void;
    get toolboxPanels(): ToolboxPanel[];
    getState(): FreehandMarkerState;
    restoreState(state: MarkerBaseState): void;
}

declare class HighlightMarker extends CoverMarker {
    static title: string;
    static icon: string;
    protected opacityPanel: OpacityPanel;
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    protected setOpacity(opacity: number): void;
    get toolboxPanels(): ToolboxPanel[];
    getState(): RectangleMarkerState;
}

declare class MeasurementMarker extends LineMarker {
    static typeName: string;
    static title: string;
    static icon: string;
    private tip1;
    private tip2;
    private get tipLength();
    constructor(container: SVGGElement, overlayContainer: HTMLDivElement, settings: Settings);
    ownsTarget(el: EventTarget): boolean;
    private createTips;
    pointerDown(point: IPoint, target?: EventTarget): void;
    protected adjustVisual(): void;
    get toolboxPanels(): ToolboxPanel[];
    getState(): LineMarkerState;
    restoreState(state: MarkerBaseState): void;
}

export { Activator, ArrowMarker, ArrowMarkerState, ArrowType, ArrowTypeChangeHandler, ArrowTypePanel, CalloutMarker, CalloutMarkerState, ColorChangeHandler, ColorPickerPanel, ColorSet, CoverMarker, DisplayMode, EllipseMarker, FontChangeHandler, FontFamilyPanel, FrameMarker, FreehandMarker, FreehandMarkerState, HighlightMarker, IStyleSettings, ITransformMatrix, LineMarker, LineMarkerState, LineStylePanel, LineWidthPanel, LinearMarkerBase, LinearMarkerBaseState, MarkerArea, MarkerAreaState, MarkerBase, MarkerBaseState, MeasurementMarker, OpacityChangeHandler, OpacityPanel, RectangleMarker, RectangleMarkerState, RectangularBoxMarkerBase, RectangularBoxMarkerBaseState, RectangularBoxMarkerGrips, ResizeGrip, Settings, Style, StyleChangeHandler, StyleClass, SvgHelper, TextMarker, TextMarkerState, ToolboxPanel, TransformMatrix, WidthChangeHandler };
